@@ -22,7 +22,9 @@
  */
 
 #include "headergenerator.h"
+#include <typedatabase.h>
 #include <reporthandler.h>
+#include <fileout.h>
 
 #include <QtCore/QDir>
 #include <QtCore/QTextStream>
@@ -65,8 +67,7 @@ void HeaderGenerator::generateClass(QTextStream& s, const AbstractMetaClass* met
     s << "#include <shiboken.h>" << endl << endl;
 
     //Includes
-    if (metaClass->typeEntry()->include().isValid())
-        s << metaClass->typeEntry()->include().toString() << endl << endl;
+    s << metaClass->typeEntry()->include() << endl;
 
     if (shouldGenerateCppWrapper(metaClass)) {
 
@@ -147,8 +148,6 @@ void HeaderGenerator::writeTypeCheckMacro(QTextStream& s, const TypeEntry* type)
     QString checkFunction = cpythonCheckFunction(type);
     s << "#define " << checkFunction << "(op) PyObject_TypeCheck(op, (PyTypeObject*)";
     s << pyTypeName << ')' << endl;
-    s << "#define " << checkFunction << "Exact(op) ((op)->ob_type == (PyTypeObject*)";
-    s << pyTypeName << ')' << endl;
 }
 
 void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* type)
@@ -193,7 +192,7 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
     if (isAbstractOrObjectType) {
         s << endl << "template<>" << endl;
         s << "struct Converter<" << type->name() << "& > : ObjectTypeReferenceConverter<" << type->name() << " >" << endl << '{' << endl;
-        s << "};" << endl << endl;
+        s << "};" << endl;
     }
 }
 
@@ -218,8 +217,7 @@ void HeaderGenerator::writeTypeIndexDefine(QTextStream& s, const AbstractMetaCla
 {
     if (!metaClass->typeEntry()->generateCode())
         return;
-    if (!metaClass->isNamespace())
-        writeTypeIndexDefineLine(s, metaClass->typeEntry(), idx);
+    writeTypeIndexDefineLine(s, metaClass->typeEntry(), idx);
     foreach (const AbstractMetaEnum* metaEnum, metaClass->enums())
         writeTypeIndexDefineLine(s, metaEnum->typeEntry(), idx);
 }
@@ -232,11 +230,9 @@ void HeaderGenerator::finishGeneration()
     // Generate the main header for this module.
     // This header should be included by binding modules
     // extendind on top of this one.
-    QString classIncludes;
-    QTextStream s_cin(&classIncludes);
-    QSet<QString> enumIncludes;
-    QString pythonTypeStuff;
-    QTextStream s_pts(&pythonTypeStuff);
+    QSet<Include> includes;
+    QString macros;
+    QTextStream macrosStream(&macros);
     QString convertersDecl;
     QTextStream convDecl(&convertersDecl);
     QString sbkTypeFunctions;
@@ -246,75 +242,58 @@ void HeaderGenerator::finishGeneration()
 
     Indentation indent(INDENT);
 
-    s_pts << "// Type indices" << endl;
+    macrosStream << "// Type indices" << endl;
     int idx = 0;
     foreach (const AbstractMetaClass* metaClass, classes())
-        writeTypeIndexDefine(s_pts, metaClass, idx);
+        writeTypeIndexDefine(macrosStream, metaClass, idx);
     foreach (const AbstractMetaEnum* metaEnum, globalEnums())
-        writeTypeIndexDefineLine(s_pts, metaEnum->typeEntry(), idx);
-    s_pts << "#define ";
-    s_pts.setFieldWidth(60);
-    s_pts << "SBK_"+moduleName()+"_IDX_COUNT";
-    s_pts.setFieldWidth(0);
-    s_pts << ' ' << idx << endl << endl;
-    s_pts << "// This variable stores all python types exported by this module" << endl;
-    s_pts << "extern PyTypeObject** " << cppApiVariableName() << ';' << endl << endl;
+        writeTypeIndexDefineLine(macrosStream, metaEnum->typeEntry(), idx);
+    macrosStream << "#define ";
+    macrosStream.setFieldWidth(60);
+    macrosStream << "SBK_"+moduleName()+"_IDX_COUNT";
+    macrosStream.setFieldWidth(0);
+    macrosStream << ' ' << idx << endl << endl;
+    macrosStream << "// This variable stores all python types exported by this module" << endl;
+    macrosStream << "extern PyTypeObject** " << cppApiVariableName() << ';' << endl << endl;
 
-    s_pts << "// Useful macros" << endl;
+    macrosStream << "// Macros for type check" << endl;
     foreach (const AbstractMetaEnum* cppEnum, globalEnums()) {
-        QString incFile = cppEnum->includeFile().split(QDir::separator()).takeLast();
-        if (!incFile.isEmpty())
-            enumIncludes << cppEnum->includeFile();
-        writeTypeCheckMacro(s_pts, cppEnum->typeEntry());
+        includes << cppEnum->typeEntry()->include();
+        writeTypeCheckMacro(macrosStream, cppEnum->typeEntry());
         FlagsTypeEntry* flags = cppEnum->typeEntry()->flags();
         if (flags)
-            writeTypeCheckMacro(s_pts, flags);
-        s_pts << endl;
+            writeTypeCheckMacro(macrosStream, flags);
         writeTypeConverterDecl(convDecl, cppEnum->typeEntry());
         convDecl << endl;
         writeSbkTypeFunction(typeFunctions, cppEnum);
     }
 
     foreach (AbstractMetaClass* metaClass, classes()) {
-        const TypeEntry* classType = metaClass->typeEntry();
-        if (!shouldGenerate(metaClass) || metaClass->enclosingClass() ||
-            !(classType->isObject() || classType->isValue() || classType->isNamespace()))
+        if (!shouldGenerate(metaClass))
             continue;
 
         //Includes
-        if (metaClass->typeEntry()->include().isValid())
-            s_cin << metaClass->typeEntry()->include().toString() << endl;
+        const TypeEntry* classType = metaClass->typeEntry();
+        includes << classType->include();
 
         foreach (const AbstractMetaEnum* cppEnum, metaClass->enums()) {
-            writeTypeCheckMacro(s_pts, cppEnum->typeEntry());
-            writeTypeConverterDecl(convDecl, cppEnum->typeEntry());
-            FlagsTypeEntry* flagsEntry = cppEnum->typeEntry()->flags();
+            EnumTypeEntry* enumType = cppEnum->typeEntry();
+            includes << enumType->include();
+            writeTypeCheckMacro(macrosStream, enumType);
+            writeTypeConverterDecl(convDecl, enumType);
+            FlagsTypeEntry* flagsEntry = enumType->flags();
             if (flagsEntry) {
-                writeTypeCheckMacro(s_pts, flagsEntry);
+                writeTypeCheckMacro(macrosStream, flagsEntry);
                 writeTypeConverterDecl(convDecl, flagsEntry);
             }
-            s_pts << endl;
             convDecl << endl;
             writeSbkTypeFunction(typeFunctions, cppEnum);
         }
 
         if (!metaClass->isNamespace()) {
             writeSbkTypeFunction(typeFunctions, metaClass);
-
             writeSbkCopyCppObjectFunction(convDecl, metaClass);
-
-            foreach (AbstractMetaClass* innerClass, metaClass->innerClasses()) {
-                if (shouldGenerate(innerClass)) {
-                    writeSbkCopyCppObjectFunction(convDecl, innerClass);
-                    s_cin << innerClass->typeEntry()->include().toString() << endl;
-                    writeTypeCheckMacro(s_pts, innerClass->typeEntry());
-                    writeTypeConverterDecl(convDecl, innerClass->typeEntry());
-                    writeTypeConverterImpl(convImpl, innerClass->typeEntry());
-                    convDecl << endl;
-                    writeSbkTypeFunction(typeFunctions, innerClass);
-                }
-            }
-            writeTypeCheckMacro(s_pts, classType);
+            writeTypeCheckMacro(macrosStream, classType);
             writeTypeConverterDecl(convDecl, classType);
             writeTypeConverterImpl(convImpl, classType);
             convDecl << endl;
@@ -327,89 +306,73 @@ void HeaderGenerator::finishGeneration()
 
     QString includeShield("SBK_" + moduleName().toUpper() + "_PYTHON_H");
 
-    QFile file(moduleHeaderFileName);
-    if (file.open(QFile::WriteOnly)) {
-        QTextStream s(&file);
+    FileOut file(moduleHeaderFileName);
+    QTextStream& s = file.stream;
+    // write license comment
+    s << licenseComment() << endl << endl;
 
-        // write license comment
-        s << licenseComment() << endl << endl;
+    s << "#ifndef " << includeShield << endl;
+    s << "#define " << includeShield << endl<< endl;
+    #ifndef AVOID_PROTECTED_HACK
+    s << "//workaround to access protected functions" << endl;
+    s << "#define protected public" << endl << endl;
+    #endif
 
-        s << "#ifndef " << includeShield << endl;
-        s << "#define " << includeShield << endl<< endl;
-        #ifndef AVOID_PROTECTED_HACK
-        s << "//workaround to access protected functions" << endl;
-        s << "#define protected public" << endl << endl;
-        #endif
+    s << "#include <Python.h>" << endl;
+    s << "#include <conversions.h>" << endl;
+    s << "#include <pyenum.h>" << endl;
+    s << "#include <basewrapper.h>" << endl;
+    s << "#include <bindingmanager.h>" << endl;
+    s << "#include <memory>" << endl << endl;
+    writeExportMacros(s);
 
-        s << "#include <Python.h>" << endl;
-        s << "#include <conversions.h>" << endl;
-        s << "#include <pyenum.h>" << endl;
-        s << "#include <basewrapper.h>" << endl;
-        s << "#include <bindingmanager.h>" << endl << endl;
-
-        s << "#include <memory>" << endl << endl;
-        writeExportMacros(s);
-
-        QStringList requiredTargetImports = TypeDatabase::instance()->requiredTargetImports();
-        if (!requiredTargetImports.isEmpty()) {
-            s << "// Module Includes" << endl;
-            foreach (const QString& requiredModule, requiredTargetImports)
-                s << "#include <" << getModuleHeaderFileName(requiredModule) << ">" << endl;
-            s << endl;
-        }
-
-        s << "// Class Includes" << endl;
-        s << classIncludes << endl;
-
-        if (!enumIncludes.isEmpty()) {
-            s << "// Enum Includes" << endl;
-            foreach (const QString& include, enumIncludes)
-              s << "#include <" << include << ">" << endl;
-            s << endl;
-        }
-
-        if (!primitiveTypes().isEmpty()) {
-            s << "// Conversion Includes - Primitive Types" << endl;
-            foreach (const PrimitiveTypeEntry* ptype, primitiveTypes()) {
-                if (ptype->include().isValid())
-                    s << ptype->include().toString() << endl;
-            }
-            s << endl;
-        }
-
-        if (!containerTypes().isEmpty()) {
-            s << "// Conversion Includes - Container Types" << endl;
-            foreach (const ContainerTypeEntry* ctype, containerTypes()) {
-                if (ctype->include().isValid())
-                    s << ctype->include().toString() << endl;
-            }
-            s << endl;
-        }
-
-        s << "extern \"C\"" << endl << '{' << endl << endl;
-        s << pythonTypeStuff << endl;
-        s << "} // extern \"C\"" << endl << endl;
-
-        s << "namespace Shiboken" << endl << '{' << endl << endl;
-
-        s << "// PyType functions, to get the PyObjectType for a type T\n";
-        s << sbkTypeFunctions << endl;
-        s << "// Generated converters declarations ----------------------------------" << endl << endl;
-        s << convertersDecl << endl;
-        s << "} // namespace Shiboken" << endl << endl;
-
-        s << "// User defined converters --------------------------------------------" << endl;
-        foreach (TypeEntry* typeEntry, TypeDatabase::instance()->entries()) {
-            if (typeEntry->hasConversionRule()) {
-                s << "// Conversion rule for: " << typeEntry->name() << endl;
-                s << typeEntry->conversionRule();
-            }
-        }
-        s << "// Generated converters implemantations -------------------------------" << endl << endl;
-        s << converterImpl << endl;
-
-        s << "#endif // " << includeShield << endl << endl;
+    QStringList requiredTargetImports = TypeDatabase::instance()->requiredTargetImports();
+    if (!requiredTargetImports.isEmpty()) {
+        s << "// Module Includes" << endl;
+        foreach (const QString& requiredModule, requiredTargetImports)
+            s << "#include <" << getModuleHeaderFileName(requiredModule) << ">" << endl;
+        s << endl;
     }
+
+    s << "// Binded library includes" << endl;
+    foreach (const Include& include, includes)
+        s << include;
+
+    if (!primitiveTypes().isEmpty()) {
+        s << "// Conversion Includes - Primitive Types" << endl;
+        foreach (const PrimitiveTypeEntry* ptype, primitiveTypes())
+            s << ptype->include();
+        s << endl;
+    }
+
+    if (!containerTypes().isEmpty()) {
+        s << "// Conversion Includes - Container Types" << endl;
+        foreach (const ContainerTypeEntry* ctype, containerTypes())
+            s << ctype->include();
+        s << endl;
+    }
+
+    s << macros << endl;
+
+    s << "namespace Shiboken" << endl << '{' << endl << endl;
+
+    s << "// PyType functions, to get the PyObjectType for a type T\n";
+    s << sbkTypeFunctions << endl;
+    s << "// Generated converters declarations ----------------------------------" << endl << endl;
+    s << convertersDecl;
+    s << "} // namespace Shiboken" << endl << endl;
+
+    s << "// User defined converters --------------------------------------------" << endl;
+    foreach (TypeEntry* typeEntry, TypeDatabase::instance()->entries()) {
+        if (typeEntry->hasConversionRule()) {
+            s << "// Conversion rule for: " << typeEntry->name() << endl;
+            s << typeEntry->conversionRule();
+        }
+    }
+    s << "// Generated converters implemantations -------------------------------" << endl << endl;
+    s << converterImpl << endl;
+
+    s << "#endif // " << includeShield << endl << endl;
 }
 
 
@@ -434,19 +397,19 @@ void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaEnu
     QString enumPrefix;
     if (cppEnum->enclosingClass())
         enumPrefix = cppEnum->enclosingClass()->qualifiedCppName() + "::";
-    s <<  "template<>\ninline PyTypeObject* SbkType<" << enumPrefix << cppEnum->name() << " >() "
+    s <<  "template<> inline PyTypeObject* SbkType<" << enumPrefix << cppEnum->name() << " >() "
       << "{ return " << cpythonTypeNameExt(cppEnum->typeEntry()) << "; }\n";
 
     FlagsTypeEntry* flag = cppEnum->typeEntry()->flags();
     if (flag) {
-        s <<  "template<>\ninline PyTypeObject* SbkType<" << flag->name() << " >() "
+        s <<  "template<> inline PyTypeObject* SbkType<" << flag->name() << " >() "
           << "{ return " << cpythonTypeNameExt(flag) << "; }\n";
     }
 }
 
 void HeaderGenerator::writeSbkTypeFunction(QTextStream& s, const AbstractMetaClass* cppClass)
 {
-    s <<  "template<>\ninline PyTypeObject* SbkType<" << cppClass->qualifiedCppName() << " >() "
+    s <<  "template<> inline PyTypeObject* SbkType<" << cppClass->qualifiedCppName() << " >() "
       <<  "{ return reinterpret_cast<PyTypeObject*>(" << cpythonTypeNameExt(cppClass->typeEntry()) << "); }\n";
 }
 
