@@ -24,6 +24,7 @@
 #include "shibokengenerator.h"
 #include "overloaddata.h"
 #include <reporthandler.h>
+#include <typedatabase.h>
 
 #include <QtCore/QDir>
 #include <QtCore/QDebug>
@@ -157,17 +158,10 @@ QString ShibokenGenerator::translateTypeForWrapperMethod(const AbstractMetaType*
     QString result;
     const TypeEntry* tentry = cType->typeEntry();
 
-    if (tentry->isValue() || tentry->isObject() || (cType->isReference() && !cType->isContainer())) {
-        result = tentry->qualifiedCppName();
-        if (cType->isReference())
-            result.append('&');
-        else if (tentry->isObject() || cType->isValuePointer())
-            result.append('*');
-    } else if (cType->isArray()) {
+    if (cType->isArray())
         result = translateTypeForWrapperMethod(cType->arrayElementType(), context) + "[]";
-    } else {
+    else
         result = translateType(cType, context);
-    }
 
     return result;
 }
@@ -310,7 +304,7 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const TypeEntry* typ
 }
 
 void ShibokenGenerator::writeBaseConversion(QTextStream& s, const AbstractMetaType* type,
-                                            const AbstractMetaClass* context)
+                                            const AbstractMetaClass* context, Options options)
 {
     QString typeName;
     if (type->isPrimitive()) {
@@ -333,7 +327,7 @@ void ShibokenGenerator::writeBaseConversion(QTextStream& s, const AbstractMetaTy
     if (typeName.startsWith("const ") && type->name() != "char")
         typeName.remove(0, sizeof("const ") / sizeof(char) - 1);
 
-    if (typeName.endsWith("&") && (tentry->isPrimitive() || tentry->isContainer()))
+    if ((options & ExcludeReference || tentry->isPrimitive() || tentry->isContainer()) && typeName.endsWith('&'))
         typeName.chop(1);
 
     s << baseConversionString(typeName);
@@ -360,29 +354,32 @@ void ShibokenGenerator::writeToCppConversion(QTextStream& s, const AbstractMetaC
 }
 
 void ShibokenGenerator::writeToCppConversion(QTextStream& s, const AbstractMetaType* type,
-                                             const AbstractMetaClass* context, const QString& argumentName)
+                                             const AbstractMetaClass* context, const QString& argumentName,
+                                             Options options)
 {
-    writeBaseConversion(s, type, context);
+    writeBaseConversion(s, type, context, options);
     s << "toCpp(" << argumentName << ')';
 }
 
-QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func) const
+QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func, bool incRef) const
 {
     QString result;
+    const char objType = (incRef ? 'O' : 'S');
     foreach (const AbstractMetaArgument* arg, func->arguments()) {
         if (func->argumentRemoved(arg->argumentIndex() + 1))
             continue;
 
         if (!func->typeReplaced(arg->argumentIndex() + 1).isEmpty()) {
-            result += 'O';
+            result += objType;
         } else if (arg->type()->isQObject()
             || arg->type()->isObject()
             || arg->type()->isValue()
             || arg->type()->isValuePointer()
             || arg->type()->isEnum()
             || arg->type()->isFlags()
+            || arg->type()->isContainer()
             || arg->type()->isReference()) {
-            result += 'O';
+            result += objType;
         } else if (arg->type()->isPrimitive()) {
             const PrimitiveTypeEntry* ptype = (const PrimitiveTypeEntry*) arg->type()->typeEntry();
             if (ptype->basicAliasedTypeEntry())
@@ -390,7 +387,7 @@ QString ShibokenGenerator::getFormatUnitString(const AbstractMetaFunction* func)
             if (m_formatUnits.contains(ptype->name()))
                 result += m_formatUnits[ptype->name()];
             else
-                result += 'O';
+                result += objType;
         } else if (isCString(arg->type())) {
             result += 'z';
         } else {
